@@ -223,9 +223,8 @@ sent_tcp(void *arg, struct tcp_pcb *pcb, u16_t len)
   struct netconn *conn;
 
   conn = arg;
-  if (conn != NULL && conn->sem != SYS_SEM_NULL) {
+  if (conn != NULL && conn->sem != SYS_SEM_NULL) 
     sys_sem_signal(conn->sem);
-  }
 
   if (conn && conn->callback)
       if (tcp_sndbuf(conn->pcb.tcp) > TCP_SNDLOWAT)
@@ -298,6 +297,10 @@ accept_function(void *arg, struct tcp_pcb *newpcb, err_t err)
 	  //printf("NO mem?\n");
     return ERR_MEM;
   }
+  
+  /* FIX MULTISTACK: newpcb->stack == conn->stack */
+  newconn->stack = newpcb->stack;  
+  
   newconn->type = NETCONN_TCP;
   newconn->pcb.tcp = newpcb;
   setup_tcp(newconn);
@@ -322,23 +325,15 @@ accept_function(void *arg, struct tcp_pcb *newpcb, err_t err)
   newconn->acceptmbox = SYS_MBOX_NULL;
   newconn->err = err;
   newconn->recv_avail = 0;
-  newconn->socket = 0;
+  newconn->socket = conn->socket;
+	newconn->callback = conn->callback;
+	if (newconn->callback) 
+		(*newconn->callback)(newconn, NETCONN_EVT_ACCEPTPLUS, 0);
   //printf("conn->acceptmbox post!\n");
   sys_mbox_post(mbox, newconn);
   /* Register event with callback */
   if (conn->callback)
-  {
     (*conn->callback)(conn, NETCONN_EVT_RCVPLUS, 0);
-    /* We have to set the callback here even though
-     * the new socket is unknown. Mark the socket as -1. */
-    newconn->callback = conn->callback;
-    /* old version: count of lost messages into socket (negative)
-     * it should not been needed with async accept */
-    /* printf("----marked socket -1\n"); */
-     newconn->socket = -1;
-     /* * newconn->socket =0; */
-  }
-  
   return ERR_OK;
 }
 #endif /* LWIP_TCP */
@@ -360,13 +355,13 @@ do_newconn(struct api_msg_msg *msg)
    switch(msg->conn->type) {
 #if LWIP_RAW
    case NETCONN_RAW:
-      msg->conn->pcb.raw = raw_new(msg->msg.bc.port); /* misusing the port field */
+      msg->conn->pcb.raw = raw_new(msg->conn->stack, msg->msg.bc.port); /* misusing the port field */
       raw_recv(msg->conn->pcb.raw, recv_raw, msg->conn);
      break;
 #endif
 #if LWIP_UDP
    case NETCONN_UDPLITE:
-      msg->conn->pcb.udp = udp_new();
+      msg->conn->pcb.udp = udp_new(msg->conn->stack);
       if(msg->conn->pcb.udp == NULL) {
          msg->conn->err = ERR_MEM;
          break;
@@ -375,7 +370,7 @@ do_newconn(struct api_msg_msg *msg)
       udp_recv(msg->conn->pcb.udp, recv_udp, msg->conn);
       break;
    case NETCONN_UDPNOCHKSUM:
-      msg->conn->pcb.udp = udp_new();
+      msg->conn->pcb.udp = udp_new(msg->conn->stack);
       if(msg->conn->pcb.udp == NULL) {
          msg->conn->err = ERR_MEM;
          break;
@@ -384,7 +379,7 @@ do_newconn(struct api_msg_msg *msg)
       udp_recv(msg->conn->pcb.udp, recv_udp, msg->conn);
       break;
    case NETCONN_UDP:
-      msg->conn->pcb.udp = udp_new();
+      msg->conn->pcb.udp = udp_new(msg->conn->stack);
       if(msg->conn->pcb.udp == NULL) {
          msg->conn->err = ERR_MEM;
          break;
@@ -394,7 +389,7 @@ do_newconn(struct api_msg_msg *msg)
 #endif /* LWIP_UDP */
 #if LWIP_TCP
    case NETCONN_TCP:
-      msg->conn->pcb.tcp = tcp_new();
+      msg->conn->pcb.tcp = tcp_new(msg->conn->stack);
       if(msg->conn->pcb.tcp == NULL) {
          msg->conn->err = ERR_MEM;
          break;
@@ -402,8 +397,8 @@ do_newconn(struct api_msg_msg *msg)
       setup_tcp(msg->conn);
       break;
 #endif
-	 default:
-			break;
+    default:
+      break;
    }
    
   
@@ -478,39 +473,39 @@ do_bind(struct api_msg_msg *msg)
     switch (msg->conn->type) {
 #if LWIP_RAW
     case NETCONN_RAW:
-      msg->conn->pcb.raw = raw_new(msg->msg.bc.port); /* misusing the port field as protocol */
+      msg->conn->pcb.raw = raw_new(msg->conn->stack, msg->msg.bc.port); /* misusing the port field as protocol */
       raw_recv(msg->conn->pcb.raw, recv_raw, msg->conn);
       break;
 #endif
 #if LWIP_PACKET
     case NETCONN_PACKET_RAW:
-      msg->conn->pcb.raw = packet_new(msg->msg.bc.port,0); /* misusing the port field as protocol */
+      msg->conn->pcb.raw = packet_new(msg->conn->stack, msg->msg.bc.port,0); /* misusing the port field as protocol */
       raw_recv(msg->conn->pcb.raw, recv_packet, msg->conn);
       break;
     case NETCONN_PACKET_DGRAM:
-      msg->conn->pcb.raw = packet_new(msg->msg.bc.port,1); /* misusing the port field as protocol */
+      msg->conn->pcb.raw = packet_new(msg->conn->stack, msg->msg.bc.port,1); /* misusing the port field as protocol */
       raw_recv(msg->conn->pcb.raw, recv_packet, msg->conn);
       break;
 #endif
 #if LWIP_UDP
     case NETCONN_UDPLITE:
-      msg->conn->pcb.udp = udp_new();
+      msg->conn->pcb.udp = udp_new(msg->conn->stack);
       udp_setflags(msg->conn->pcb.udp, UDP_FLAGS_UDPLITE);
       udp_recv(msg->conn->pcb.udp, recv_udp, msg->conn);
       break;
     case NETCONN_UDPNOCHKSUM:
-      msg->conn->pcb.udp = udp_new();
+      msg->conn->pcb.udp = udp_new(msg->conn->stack);
       udp_setflags(msg->conn->pcb.udp, UDP_FLAGS_NOCHKSUM);
       udp_recv(msg->conn->pcb.udp, recv_udp, msg->conn);
       break;
     case NETCONN_UDP:
-      msg->conn->pcb.udp = udp_new();
+      msg->conn->pcb.udp = udp_new(msg->conn->stack);
       udp_recv(msg->conn->pcb.udp, recv_udp, msg->conn);
       break;
 #endif /* LWIP_UDP */
 #if LWIP_TCP      
     case NETCONN_TCP:
-      msg->conn->pcb.tcp = tcp_new();
+      msg->conn->pcb.tcp = tcp_new(msg->conn->stack);
       setup_tcp(msg->conn);
 #endif /* LWIP_TCP */
     default:  
@@ -578,13 +573,13 @@ do_connect(struct api_msg_msg *msg)
     switch (msg->conn->type) {
 #if LWIP_RAW
     case NETCONN_RAW:
-      msg->conn->pcb.raw = raw_new(msg->msg.bc.port); /* misusing the port field as protocol */
+      msg->conn->pcb.raw = raw_new(msg->conn->stack, msg->msg.bc.port); /* misusing the port field as protocol */
       raw_recv(msg->conn->pcb.raw, recv_raw, msg->conn);
       break;
 #endif
 #if LWIP_UDP
     case NETCONN_UDPLITE:
-      msg->conn->pcb.udp = udp_new();
+      msg->conn->pcb.udp = udp_new(msg->conn->stack);
       if (msg->conn->pcb.udp == NULL) {
 				msg->conn->err = ERR_MEM;
 				sys_mbox_post(msg->conn->mbox, NULL);
@@ -594,7 +589,7 @@ do_connect(struct api_msg_msg *msg)
       udp_recv(msg->conn->pcb.udp, recv_udp, msg->conn);
       break;
     case NETCONN_UDPNOCHKSUM:
-      msg->conn->pcb.udp = udp_new();
+      msg->conn->pcb.udp = udp_new(msg->conn->stack);
       if (msg->conn->pcb.udp == NULL) {
 				msg->conn->err = ERR_MEM;
 				sys_mbox_post(msg->conn->mbox, NULL);
@@ -604,7 +599,7 @@ do_connect(struct api_msg_msg *msg)
       udp_recv(msg->conn->pcb.udp, recv_udp, msg->conn);
       break;
     case NETCONN_UDP:
-      msg->conn->pcb.udp = udp_new();
+      msg->conn->pcb.udp = udp_new(msg->conn->stack);
       if (msg->conn->pcb.udp == NULL) {
 				msg->conn->err = ERR_MEM;
 				sys_mbox_post(msg->conn->mbox, NULL);
@@ -615,7 +610,7 @@ do_connect(struct api_msg_msg *msg)
 #endif /* LWIP_UDP */
 #if LWIP_TCP      
     case NETCONN_TCP:
-      msg->conn->pcb.tcp = tcp_new();      
+      msg->conn->pcb.tcp = tcp_new(msg->conn->stack);      
       if (msg->conn->pcb.tcp == NULL) {
 				msg->conn->err = ERR_MEM;
 				sys_mbox_post(msg->conn->mbox, NULL);
@@ -914,9 +909,9 @@ api_msg_input(struct api_msg *msg)
 }
 
 void
-api_msg_post(struct api_msg *msg)
+api_msg_post(struct stack *stack, struct api_msg *msg)
 {
-  tcpip_apimsg(msg);
+  tcpip_apimsg(stack, msg);
 }
 
 
