@@ -22,14 +22,18 @@
 
 #if IPv6_AUTO_CONFIGURATION
 
+#include "lwip/debug.h"
+#include "lwip/def.h"
 #include "lwip/sys.h"
 #include "lwip/netif.h"
 #include "lwip/ip.h"
 #include "lwip/ip_addr.h"
 
 #include "lwip/icmp.h"
+
 #include "lwip/ip_autoconf.h"
 
+
 #include "lwip/netlinkdefs.h"
 
 
@@ -49,11 +53,19 @@ void start_dad_on_addr(struct ip_addr_list *addr, struct netif *netif);
 /*--------------------------------------------------------------------------*/
 
 void 
-ip_autoconf_init(void)
+ip_autoconf_init(struct stack *stack)
 {
 	LWIP_DEBUGF(IP_AUTOCONF_DEBUG, ("%s: IPv6 Address Autoconfiguration Enabled.\n", __func__));	
 }
 
+void
+ip_autoconf_shutdown(struct stack *stack)
+{
+	/* FIX: TODO*/
+
+	LWIP_DEBUGF(IP_AUTOCONF_DEBUG, ("%s: IPv6 Address Autoconfiguration stopped.\n", __func__));	
+}
+
 
 /* Initialize interface data for autoconfiguration protocol */
 void 
@@ -110,6 +122,8 @@ ip_autoconf_handle_ra(struct netif *netif, struct pbuf *p, struct ip_hdr *iphdr,
 	struct ip_addr ipaddr;
 	struct ip_addr netmask;
 	struct ip_addr_list *confaddr;
+	
+	struct stack *stack = netif->stack;
 
 	LWIP_DEBUGF(IP_AUTOCONF_DEBUG, ("%s: Processing received RA \n", __func__) );
 
@@ -156,7 +170,7 @@ ip_autoconf_handle_ra(struct netif *netif, struct pbuf *p, struct ip_hdr *iphdr,
 		/* TODO: check if the entry already exists and implement route lifetime */
                 IP6_ADDR_UNSPECIFIED(&ipaddr);
                 IP6_ADDR_UNSPECIFIED(&netmask);
-		ip_route_list_add(&ipaddr,&netmask, &iphdr->src, netif, 0);
+		ip_route_list_add(stack, &ipaddr,&netmask, &iphdr->src, netif, 0);
 	}
 
 	LWIP_DEBUGF(IP_AUTOCONF_DEBUG, ("\tHop-Lim:%u ", ira->hoplimit));
@@ -215,7 +229,7 @@ ip_autoconf_handle_ra(struct netif *netif, struct pbuf *p, struct ip_hdr *iphdr,
 		/* We can use this prefix for autoconfiguration */
 		if (oprefix->flags & ICMP6_OPT_PREF_A) {
 
-			confaddr = ip_addr_list_alloc();
+			confaddr = ip_addr_list_alloc(stack);
 			if (confaddr != NULL) {
 
 				confaddr->flags = IFA_F_TENTATIVE;
@@ -248,7 +262,7 @@ ip_autoconf_handle_ra(struct netif *netif, struct pbuf *p, struct ip_hdr *iphdr,
 						ip_addr_set(&ipaddr, &(confaddr->ipaddr));
 						ip_addr_set(&netmask, &(confaddr->netmask));
 
-						ip_route_list_add(&ipaddr, &netmask, NULL, netif, 0);
+						ip_route_list_add(stack, &ipaddr, &netmask, NULL, netif, 0);
 					}
 			
 					netif->autoconf.status = AUTOCONF_SUCC;
@@ -259,7 +273,7 @@ ip_autoconf_handle_ra(struct netif *netif, struct pbuf *p, struct ip_hdr *iphdr,
 				else {
 					/* Discard autogenerated packet */
 					LWIP_DEBUGF(IP_AUTOCONF_DEBUG, ("\tAddress already added.\n"));
-					ip_addr_list_free(confaddr);
+					ip_addr_list_free(stack, confaddr);
 
 					/* TODO: update (address|route entry)'s life-time information? */
 				}
@@ -277,6 +291,9 @@ ip_autoconf_handle_na(struct netif *netif, struct pbuf *p, struct ip_hdr *iphdr,
 {
 	struct ip_addr_list * ip;
 
+	struct stack *stack = netif->stack;
+
+
 	LWIP_DEBUGF(IP_AUTOCONF_DEBUG, ("%s: Processing received NA \n", __func__) );
 
 
@@ -310,7 +327,7 @@ ip_autoconf_handle_na(struct netif *netif, struct pbuf *p, struct ip_hdr *iphdr,
 
 					/* This address is duplicated, remove it */
 					ip_addr_list_del(&netif->autoconf.addrs_tentative, ip);
-					ip_addr_list_free(ip);
+					ip_addr_list_free(stack, ip);
 
 					/* FIX: remove any routing table's entries ???? */
 
@@ -350,6 +367,8 @@ dad_timeout(void *arg)
 {
 	struct ip_addr_list *addr = (struct ip_addr_list *) arg;
 	struct netif * netif = addr->netif;
+	
+	struct stack *stack = netif->stack;
 
 	/* FIX: LOCK netif->addr */
 
@@ -362,7 +381,8 @@ dad_timeout(void *arg)
 		/* More probes to send? */
 		if (addr->info.dad_counter < addr->netif->autoconf.dad_duptrans) {
 			LWIP_DEBUGF(IP_AUTOCONF_DEBUG, ("%s: still TENTATIVE (probes %d/%d)\n", __func__, addr->info.dad_counter, addr->netif->autoconf.dad_duptrans));  
-			icmp_send_dad(addr, netif);
+			
+			icmp_send_dad(stack, addr, netif);
 			addr->info.dad_counter++;
 
 			/* Set next timeout */
@@ -395,6 +415,8 @@ dad_remove_timer(struct ip_addr_list *addr)
 void 
 start_dad_on_addr(struct ip_addr_list *addr, struct netif *netif)
 {
+	struct stack *stack = netif->stack;
+	
 	LWIP_DEBUGF(IP_AUTOCONF_DEBUG, ("%s: start on ", __func__));                        
 	ip_addr_debug_print(IP_AUTOCONF_DEBUG, &(addr->ipaddr));
 	LWIP_DEBUGF(IP_AUTOCONF_DEBUG, ("\n"));                        
@@ -403,7 +425,7 @@ start_dad_on_addr(struct ip_addr_list *addr, struct netif *netif)
 	addr->info.flag = IPADDR_TENTATIVE;
 	addr->info.dad_counter = 0;
 
-	icmp_send_dad(addr, netif);
+	icmp_send_dad(stack, addr, netif);
 
 	/* Set DAD timeout for this entry. */
 	addr->info.dad_counter++;
@@ -426,6 +448,8 @@ rtr_sol_timeout(void *arg)
 	struct ip_addr_list unspecified;
 	struct ip_addr      linkscope;
 	struct ip_addr_list *srcaddr;
+	
+	struct stack *stack = netif->stack;
 
 	/* If we received a Router Advertisment then netif->autoconf.status != INIT and
 	   stop protocol */
@@ -448,7 +472,8 @@ rtr_sol_timeout(void *arg)
 		}
 
 		/* Send RS */
-		icmp_router_solicitation(NULL, srcaddr);
+		icmp_router_solicitation(stack, NULL, srcaddr);
+		
 		netif->autoconf.rtr_sol_counter++;
 
 		/* Set next timeout */
@@ -486,7 +511,7 @@ start_router_solicitation(struct netif *netif)
 /* Update lifetime and returns 1 if there is a valid (or tentative) link-scope 
    address in the list */
 INLINE static int 
-addr_update_lifetime(struct ip_addr_list **addrs, u32_t time)
+addr_update_lifetime(struct stack *stack, struct ip_addr_list **addrs, u32_t time)
 {
 	struct ip_addr_list *cur, *next;
 	struct ip_addr_list *list, *new_list;
@@ -536,8 +561,9 @@ addr_update_lifetime(struct ip_addr_list **addrs, u32_t time)
 				LWIP_DEBUGF(IP_AUTOCONF_DEBUG, ("%s: address expired! ", __func__));                        
 				ip_addr_debug_print(IP_AUTOCONF_DEBUG, &(cur->ipaddr));
 				LWIP_DEBUGF(IP_AUTOCONF_DEBUG, ("\n"));                        
+				
 				/* TODO: Remove this address, close connections, ecc... */
-				ip_addr_list_free(cur);
+				ip_addr_list_free(stack, cur);
 			}
 		}
 
@@ -554,9 +580,11 @@ addr_update_lifetime(struct ip_addr_list **addrs, u32_t time)
 static struct ip_addr_list * 
 create_link_scope_addr(struct netif *netif)
 {
+	struct stack *stack = netif->stack;
+	
 	struct ip_addr_list *add = NULL;
 	
-	add = ip_addr_list_alloc();
+	add = ip_addr_list_alloc(stack);
 	if (add != NULL) {
 
 		IP6_ADDR_LINKSCOPE(&(add->ipaddr), netif->hwaddr);
@@ -581,6 +609,8 @@ create_link_scope_addr(struct netif *netif)
 static void 
 remove_autoconf_data(struct netif *netif)
 {
+	struct stack *stack = netif->stack;
+	
 	struct ip_addr_list *cur, *next;
 	struct ip_addr_list *list, *new_list;
 
@@ -609,7 +639,7 @@ remove_autoconf_data(struct netif *netif)
 				/* FIX: ABORT TCP,UDP,ICMP connections */
 				//ip_addr_close(&(cur->ipaddr));
 
-				ip_addr_list_free(cur);
+				ip_addr_list_free(stack, cur);
 			}
 	
 			cur = next;
@@ -647,14 +677,16 @@ ip_autoconf_timer(void *arg)
 
 	struct netif *netif = (struct netif *) arg;
 
+	struct stack *stack = netif->stack;
+
 	LWIP_DEBUGF(IP_AUTOCONF_DEBUG, ("%s: start on %c%c%d \n", __func__,	
 		netif->name[0], netif->name[1], netif->num));
 
 	/* Update addresses status (and remove invalid addresses) */
-	have_linkaddr =  addr_update_lifetime(&netif->autoconf.addrs_tentative, (AUTOCONF_TMR_INTERVAL/1000));
+	have_linkaddr =  addr_update_lifetime(stack, &netif->autoconf.addrs_tentative, (AUTOCONF_TMR_INTERVAL/1000));
 
 	/* FIX: LOCK netif->addrs */
-	have_linkaddr |= addr_update_lifetime(&netif->addrs, (AUTOCONF_TMR_INTERVAL/1000));
+	have_linkaddr |= addr_update_lifetime(stack, &netif->addrs, (AUTOCONF_TMR_INTERVAL/1000));
 	/* FIX: UNLOCK netif->addrs */
 
 	sys_timeout(AUTOCONF_TMR_INTERVAL, ip_autoconf_timer  , netif);
@@ -664,6 +696,8 @@ ip_autoconf_timer(void *arg)
 void 
 ip_autoconf_start(struct netif *netif)
 {
+	struct stack *stack = netif->stack;
+	
 	struct ip_addr_list *linkadd;
 
 	LWIP_DEBUGF(IP_AUTOCONF_DEBUG, ("%s: start.\n", __func__) );
@@ -681,7 +715,7 @@ ip_autoconf_start(struct netif *netif)
 			ip_addr_list_add(&(netif->autoconf.addrs_tentative),linkadd);
 
 			/* Add routing entry for link-scope addresses */
-			ip_route_list_add(&(linkadd->ipaddr),&(linkadd->netmask), NULL, netif, 0);
+			ip_route_list_add(stack, &(linkadd->ipaddr),&(linkadd->netmask), NULL, netif, 0);
 
 			/* Start DAD on link-scope address */
 			start_dad_on_addr(linkadd, netif);
