@@ -54,6 +54,7 @@
 
 #include "lwip/opt.h"
 #include "lwip/arch.h"
+struct stack;
 
 
 struct ip_addr {
@@ -97,10 +98,10 @@ typedef struct {
 ///};
 
 
-#define IP6_ADDR(ipaddr, a,b,c,d,e,f,g,h) do { (ipaddr)->addr[0] = htonl((u32_t)((a & 0xffff) << 16) | (b & 0xffff)); \
-                                               (ipaddr)->addr[1] = htonl(((c & 0xffff) << 16) | (d & 0xffff)); \
-                                               (ipaddr)->addr[2] = htonl(((e & 0xffff) << 16) | (f & 0xffff)); \
-                                               (ipaddr)->addr[3] = htonl(((g & 0xffff) << 16) | (h & 0xffff)); } while(0)
+#define IP6_ADDR(ipaddr, a,b,c,d,e,f,g,h) do { (ipaddr)->addr[0] = htonl((u32_t)(((a) & 0xffff) << 16) | ((b) & 0xffff)); \
+                                               (ipaddr)->addr[1] = htonl((((c) & 0xffff) << 16) | ((d) & 0xffff)); \
+                                               (ipaddr)->addr[2] = htonl((((e) & 0xffff) << 16) | ((f) & 0xffff)); \
+                                               (ipaddr)->addr[3] = htonl((((g) & 0xffff) << 16) | ((h) & 0xffff)); } while(0)
 
 
 #define IP4_ADDR(ip4addr, a,b,c,d) (ip4addr)->addr = htonl(((u32_t)((a) & 0xff) << 24) | ((u32_t)((b) & 0xff) << 16) | \
@@ -304,13 +305,16 @@ void ip_addr_list_add(struct ip_addr_list **ptail, struct ip_addr_list *el);
 
 void ip_addr_list_del(struct ip_addr_list **ptail, struct ip_addr_list *el);
 
-void ip_addr_list_init();
 
-struct ip_addr_list *ip_addr_list_alloc();
+void ip_addr_list_init(struct stack *stack);
+void ip_addr_list_shutdown(struct stack *stack);
 
-void ip_addr_list_free(struct ip_addr_list *el);
 
-void ip_addr_list_freelist(struct ip_addr_list *tail);
+struct ip_addr_list *ip_addr_list_alloc(struct stack *stack);
+
+void ip_addr_list_free(struct stack *stack, struct ip_addr_list *el);
+
+void ip_addr_list_freelist(struct stack *stack, struct ip_addr_list *tail);
 
 struct ip_addr_list *ip_addr_list_find(struct ip_addr_list *tail, struct ip_addr *addr, struct ip_addr *netmask);
 
