@@ -55,7 +55,7 @@
 #define __LWIP_SOCKETS_H__
 
 #include "lwip/ip_addr.h"
-
+struct stack;
 struct in_addr {
           u32_t s_addr;
 };
@@ -90,6 +90,16 @@ struct sockaddr {
 #  define socklen_t int
 #endif
 
+struct msghdr {
+	void         *msg_name;       /* optional address */
+	socklen_t     msg_namelen;    /* size of address */
+	struct iovec *msg_iov;        /* scatter/gather array */
+	size_t        msg_iovlen;     /* # elements in msg_iov */
+	void         *msg_control;    /* ancillary data, see below */
+	socklen_t     msg_controllen; /* ancillary data buffer len */
+	int           msg_flags;      /* flags on received message */
+};
+
 
 #define SOCK_STREAM     1
 #define SOCK_DGRAM      2
@@ -207,6 +217,11 @@ struct linger {
 #define IP_TOS       1
 #define IP_TTL       2
 #define IP_HDRINCL   3
+#define IP_MTU_DISCOVER    10 /* int */
+#define IP_RECVERR         11 /* bool */
+#define IP_RECVTTL         12 /* bool */
+#define IP_RECVTOS         13 /* bool */
+
 
 /*
  * Options for level IPPROTO_RAW
@@ -422,21 +437,23 @@ int lwip_setsockopt (int s, int level, int optname, const void *optval, socklen_
 int lwip_close(int s);
 int lwip_connect(int s, struct sockaddr *name, socklen_t namelen);
 int lwip_listen(int s, int backlog);
-int lwip_recv(int s, void *mem, int len, unsigned int flags);
-int lwip_read(int s, void *mem, int len);
-int lwip_recvfrom(int s, void *mem, int len, unsigned int flags,
+ssize_t lwip_recv(int s, void *mem, int len, unsigned int flags);
+ssize_t lwip_read(int s, void *mem, int len);
+ssize_t lwip_recvfrom(int s, void *mem, int len, unsigned int flags,
       struct sockaddr *from, socklen_t *fromlen);
-int lwip_send(int s, void *dataptr, int size, unsigned int flags);
-int lwip_sendto(int s, void *dataptr, int size, unsigned int flags,
+ssize_t lwip_recvmsg(int fd, struct msghdr *msg, int flags);
+ssize_t lwip_send(int s, void *dataptr, int size, unsigned int flags);
+ssize_t lwip_sendto(int s, void *dataptr, int size, unsigned int flags,
     struct sockaddr *to, socklen_t tolen);
+ssize_t lwip_sendmsg(int fd, struct msghdr *msg, int flags);
+int lwip_msocket(struct stack *stack, int domain, int type, int protocol);
 int lwip_socket(int domain, int type, int protocol);
-int lwip_write(int s, void *dataptr, int size);
+ssize_t lwip_write(int s, void *dataptr, int size);
 int lwip_select(int maxfdp1, fd_set *readset, fd_set *writeset, fd_set *exceptset,
                 struct timeval *timeout);
-int lwip_ioctl(int s, long cmd, void *argp);
-
-
-
+int lwip_ioctl(int s, unsigned long cmd, void *argp);
+int lwip_fcntl64(int s, int cmd, long arg);
+int lwip_fcntl(int s, int cmd, long arg);
 
 /*
  * Taken from sys/uio.h and bits/uio.h
@@ -465,10 +482,6 @@ int lwip_readv(int s, struct iovec *vector, int count);
 int lwip_writev(int s, struct iovec *vectorc, int count);
 
 
-
-
-
-
 #if LWIP_COMPAT_SOCKETS
 #define accept(a,b,c)         lwip_accept(a,b,c)
 #define bind(a,b,c)           lwip_bind(a,b,c)
@@ -493,7 +506,6 @@ int lwip_writev(int s, struct iovec *vectorc, int count);
 #define writev(a,b,c)         lwip_writev(a,b,c)
 #define readv(a,b,c)          lwip_readv(a,b,c)
 
-
 #endif /* LWIP_COMPAT_SOCKETS */
 
 #endif /* __LWIP_SOCKETS_H__ */
